
;**************************************************************************
;*                      Easy Master Basis Firmware                        *
;*                                                                        *
;*   MSSP1 is used as an I2C Master Port                                  *
;*   MSSP2 is used as an SPI Master Port                                  *
;**************************************************************************
;*  EasyMaster2018 - PCF8574 Output Operation I2C Example                 *
;**************************************************************************
;*          v 1.00 - 10.03.2018                                           *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F1829 used                                       *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************

 INCLUDE "P16F1829.INC"


_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002

;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
v_spi_buffi	EQU	H'22'
v_i2c_add	EQU	H'23'
;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	clrf	BSR
		clrf	INTCON		;disable interrupts

		banksel	OPTION_REG	;BANK 1
		movlw	B'00111111'
		movwf	OPTION_REG	;see datasheet Page 176

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA
		clrf	ANSELB
		clrf	ANSELC

		movlw	H'01'
		movwf	BSR		;select BANK 1
		movlw	B'11111111'
		movwf	TRISA
		movlw	B'11111111'
		movwf	TRISB
		movlw	B'00000000'
		movwf	TRISC

		movf	OSCCON,0
		andlw	B'11111111'
		iorlw	B'01111000'
		movwf	OSCCON		;set 16MHz internal clock (bit 6:3 = 1111) - Datasheet Page 68

		banksel	WPUC
		clrf	WPUA
		clrf	WPUB
		clrf	WPUC

		clrf	BSR		;select BANK 0
		clrf	PORTC
;------------------------------------------------------------------------------
		goto	m_application
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
dr4		call	dr2
		call	dr2
		return
;-----------------------------------
dr2		call	d2
		call	d2
		return
;-----------------------------------
d2		movlw	D'255'
		movwf	TIMER1
d2_1		movlw	D'255'
		movwf	TIMER2
d21_loop	decfsz	TIMER2,F
		goto	d21_loop
		decfsz	TIMER1,F
		goto	d2_1
		return
;----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines - new
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
i2c_start	movlw	D'255'
		movwf	TIMER2

		bcf	PIR1,SSP1IF

		banksel	SSP1CON2
		bsf	SSP1CON2,SEN
wait1		clrf	BSR
		decfsz	TIMER2,1
		goto	cont22
		goto	i2c_time_out
cont22		btfss	PIR1,SSP1IF
		goto	wait1
		bcf	PIR1,SSP1IF
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	bcf	PIR1,SSP1IF
		banksel	SSP1BUF
		movwf	SSP1BUF
		clrf	BSR
wait2		btfss	PIR1,SSP1IF
		goto	wait2
		bcf	PIR1,SSP1IF
		banksel	SSP1CON2
		btfss	SSP1CON2,ACKSTAT
		goto	ack_received
		goto	ack_not_rec
;------------------------------------------------------------------
ack_received	clrf	BSR
		bsf	PORTC,D'002'		;turn on green LED
		goto	ccc2
;------------------------------------------------------------------
ack_not_rec	clrf	BSR
		bsf	PORTC,D'000'		;turn on red LED
		goto	ccc2
;------------------------------------------------------------------
ccc2		bcf	PIR1,SSP1IF
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
xi2c_start	nop
		banksel	SSP1CON2
		bsf	SSP1CON2,SEN
		clrf	BSR
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	nop
		banksel	SSP1CON2
		bsf	SSP1CON2,PEN
		clrf	BSR
		call	d55
		banksel	LATC
		bcf	LATC,D'002'
		bcf	LATC,D'000'
		clrf	BSR
		return
;-----------------------------------------------------------------------------
i2c_ack		nop
		banksel	SSP1CON2
		bcf	SSP1CON2,ACKDT			;ACK
		bsf	SSP1CON2,ACKEN
		clrf	BSR
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	nop
		banksel	SSP1CON2
		bsf	SSP1CON2,ACKDT			;NOT ACK
		bsf	SSP1CON2,ACKEN
		clrf	BSR
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	nop
		banksel	SSP1CON2
		bsf	SSP1CON2,RCEN
		clrf	BSR
		call	d55
		banksel	SSP1BUF
		movf	SSP1BUF,0
		clrf	BSR
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
xi2c_send	nop
		banksel	SSP1BUF
		movwf	SSP1BUF
		clrf	BSR
		call	d55		;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	nop
		banksel	SSP1CON2
		bsf	SSP1CON2,RSEN
		clrf	BSR
		call	d55
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;I2C communication error - time out occured during I2C I/O operations
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
i2c_time_out	clrf	PORTC
		call	dr4
		call	dr4
		reset
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;I2C Interface initialization
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
i2c_init	nop
		banksel	SSP1CON1
;set up I2C BUS for communication as a master for MSSP1
		movlw	B'00101000'		;see Datasheet Page 260
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSP1CON1

;set-up I2C speed
		movlw	B'00111111'
		movwf	SSP1ADD
		clrf	SSP1STAT

		clrf	BSR
		bsf	PORTC,D'003'
		return
;---------------------------------------------------------------------------------
;---------------------------------------------------------------------------------


;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; SPI communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

spi_init_basic	nop
		banksel	TRISA
		bsf	TRISB,D'005'		;SDI2 (RB5 -- input)
		bcf	TRISA,D'005'		;SDO2 (RA5 -- output)
		bcf	TRISB,D'007'		;SCK2 (RB7 -- output for master mode)

		bcf	TRISA,D'004'		;CS0 (RA4 -- output)

		banksel	APFCON1
		bsf	APFCON1,SDO2SEL		;select RA5 as MOSI for MSSP2

		clrf	BSR			;select BANK0
		bsf	PORTA,D'004'		;CS = 1
		bsf	PORTC,D'001'		;switch on the "SPI-LED" (LED6)
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;set up SPI BUS for communication as a master for MSSP2 in SPI Mode 1
;-----------------------------------------------------------------------------
spi_init_mode_1	call	spi_init_basic
		banksel	SSP2CON1
		movlw	B'00000010'		;see Datasheet Page 278
						;bit 7   = 0    -> N/A
						;bit 6   = 0    -> N/A
						;bit 5   = 1    -> enables MSSP2
						;bit 4   = 0    -> Idle state for clock is a low level
						;bit 3-0 = 0010 -> SPI Master mode, clock = FOSC/64
		movwf	SSP2CON1
;
		movlw	B'01000000'
		movwf	SSP2STAT		;bit 7   = 0 -> Input data sampled at middle of data output time
						;bit 6   = 1 -> Transmit occurs on transition from Active to Idle clock state
		bsf	SSP2CON1,D'005'		;enable SPI on MSSP2
		clrf	BSR
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
spi_cs0_en	bcf	PORTA,D'004'		;CS = 0
		return		
;--------------------------------------
spi_cs0_dis	bsf	PORTA,D'004'		;CS = 1
		return		
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
spi_send	nop
		banksel	SSP2CON1
		bcf	SSP2CON1,SSPOV		;just for sure...
		bcf	SSP2STAT,BF

		movwf	SSP2BUF
spil2		btfss	SSP2STAT,BF
		goto	spil2
		movf	SSP2BUF,0
		clrf	BSR
		movwf	v_spi_buffi
		return
;-----------------------------------------------------------------------------
; 02_EasyMaster_3_6_3_5--PCF8574.asm
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;   EasyMaster2016 - PCF8574 Output Operation I2C Example - 3.6.3.5
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
m_application	bsf	PORTC,D'004'		;alive LED (LED2)
		call	i2c_init		;(1)
;------------------------------------------------------------------------------
;(A) - open the I2C connection for PCF8574
		call	i2c_start		;(1)
		movlw	B'01000000'		;(2) - device I2C address + write
		call	i2c_send
;-----------------------------------------------------------------------------
;(B) - send data in endless loop
main_loop	movlw	B'11111110'
		call	i2c_send
		call	dr2
		movlw	B'11111101'
		call	i2c_send
		call	dr2
		movlw	B'11111011'
		call	i2c_send
		call	dr2
		movlw	B'11110111'
		call	i2c_send
		call	dr2
		movlw	B'11101111'
		call	i2c_send
		call	dr2
		movlw	B'11011111'
		call	i2c_send
		call	dr2
		movlw	B'10111111'
		call	i2c_send
		call	dr2
		movlw	B'01111111'
		call	i2c_send
		call	dr2
		goto	main_loop
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;******************************************************************************
;******************************************************************************
		END